@@ -1,15 +1,13 @@
-use santiago::grammar::GrammarRule;
 use santiago::languages::nix::grammar_rules;
 use santiago::languages::nix::lexer_rules;
 use santiago::lexer::lex;
 use santiago::lexer::Lexeme;
-use santiago::lexer::LexerRule;
 use santiago::parser::parse;
-use santiago::parser::Forest;
+use santiago::parser::Tree;
 
 fn main() -> Result<(), String> {
-    let lexing_rules: Vec<LexerRule> = lexer_rules();
-    let grammar_rules: Vec<GrammarRule> = grammar_rules(&lexing_rules);
+    let lexing_rules = lexer_rules();
+    let grammar_rules = grammar_rules(&lexing_rules);
 
     let stdin = get_stdin();
     let lexemes: Vec<Lexeme> = lex(&lexing_rules, &stdin);
@@ -19,7 +17,7 @@ fn main() -> Result<(), String> {
         println!("  {lexeme}");
     }
 
-    let abstract_syntax_trees: Vec<Forest> = parse(&grammar_rules, &lexemes)?;
+    let abstract_syntax_trees: Vec<Tree> = parse(&grammar_rules, &lexemes)?;
 
     println!("AST:");
     for ast in abstract_syntax_trees {
