@@ -6,7 +6,7 @@ use crate::grammar::Symbol;
 
 use std;
 
-#[derive(Clone, Eq, Hash, PartialEq)]
+#[derive(Clone, Debug, Eq, Hash, PartialEq)]
 pub struct Production {
     pub symbols: Vec<Symbol>,
 }
