@@ -2,68 +2,89 @@
 //
 // SPDX-License-Identifier: GPL-3.0-only
 
+use crate::grammar::Grammar;
 use crate::grammar::GrammarRule;
-use crate::grammar::Production;
 use crate::grammar::Symbol;
+use crate::grammar::START_RULE_NAME;
 use crate::lexer::Lexeme;
-use crate::parser::forest::build_forest;
-use crate::parser::Forest;
+use crate::parser::tree::build_forest;
 use crate::parser::ParserColumn;
 use crate::parser::ParserState;
+use crate::parser::Tree;
 use std::collections::HashSet;
 
-const START_RULE_NAME: &str = "Γ";
-
 fn predict(column: &mut ParserColumn, rule: &GrammarRule) {
     for production in &rule.productions {
-        column.add(ParserState {
+        let state = ParserState {
             name:         rule.name.clone(),
             production:   production.clone(),
             dot_index:    0,
             start_column: column.index,
             end_column:   usize::MAX,
-        });
+        };
+        column.add(state);
     }
 }
 
-fn scan(column: &mut ParserColumn, state: &ParserState, kind: &str) {
-    if kind == column.kind {
-        column.add(ParserState {
-            name:         state.name.clone(),
-            production:   state.production.clone(),
-            start_column: state.start_column,
-            end_column:   usize::MAX,
-            dot_index:    state.dot_index + 1,
-        });
-    }
+fn scan(
+    columns: &mut Vec<ParserColumn>,
+    column_index: usize,
+    state_index: usize,
+) {
+    let state = &columns[column_index].states[state_index];
+    let new_state = ParserState {
+        name:         state.name.clone(),
+        production:   state.production.clone(),
+        start_column: state.start_column,
+        end_column:   usize::MAX,
+        dot_index:    state.dot_index + 1,
+    };
+    columns[column_index + 1].add(new_state);
 }
 
 fn complete(
     columns: &mut Vec<ParserColumn>,
     column_index: usize,
-    state: &ParserState,
+    state_index: usize,
 ) {
-    if state.completed() {
-        for st in &columns[state.start_column].states.clone() {
-            if let Some(Symbol::Rule(name)) = st.next_symbol() {
-                if name == state.name {
-                    columns[column_index].add(ParserState {
-                        name:         st.name.clone(),
-                        production:   st.production.clone(),
-                        start_column: st.start_column,
-                        end_column:   usize::MAX,
-                        dot_index:    st.dot_index + 1,
-                    });
+    let state_name = &columns[column_index].states[state_index].name;
+    let state_start_column =
+        columns[column_index].states[state_index].start_column;
+
+    let indexes: Vec<usize> = columns[state_start_column]
+        .states
+        .iter()
+        .enumerate()
+        .filter_map(|(index, st)| match st.next_symbol() {
+            Some(Symbol::Rule(name)) => {
+                if name == *state_name {
+                    Some(index)
+                } else {
+                    None
                 }
             }
-        }
+            _ => None,
+        })
+        .collect();
+
+    for index in indexes {
+        let st = &columns[state_start_column].states[index];
+        let parser_state = ParserState {
+            name:         st.name.clone(),
+            production:   st.production.clone(),
+            start_column: st.start_column,
+            end_column:   usize::MAX,
+            dot_index:    st.dot_index + 1,
+        };
+        columns[column_index].add(parser_state);
     }
 }
 
+/// Parse the provided (Lexemes)(Lexeme) with the given [Grammar]
 pub fn parse(
-    rules: &[GrammarRule],
+    grammar: &Grammar,
     lexemes: &[Lexeme],
-) -> Result<Vec<Forest>, String> {
+) -> Result<Vec<Tree>, String> {
     let mut columns: Vec<ParserColumn> = (0..=lexemes.len())
         .map(|index| {
             if index == 0 {
@@ -84,11 +105,10 @@ pub fn parse(
         })
         .collect();
 
-    columns[0].states.push(ParserState {
-        name:         START_RULE_NAME.to_string(),
-        production:   Production {
-            symbols: vec![Symbol::Rule(rules[0].name.clone())],
-        },
+    let name = START_RULE_NAME.to_string();
+    columns[0].add(ParserState {
+        name:         name.clone(),
+        production:   grammar.rules.get(&name).unwrap().productions[0].clone(),
         start_column: 0,
         end_column:   usize::MAX,
         dot_index:    0,
@@ -99,22 +119,21 @@ pub fn parse(
         let mut state_len = columns[column_index].states.len();
 
         while state_index < state_len {
-            let state = columns[column_index].states[state_index].clone();
+            let state = &columns[column_index].states[state_index];
 
-            if state.completed() {
-                complete(&mut columns, column_index, &state);
+            if columns[column_index].states[state_index].completed() {
+                complete(&mut columns, column_index, state_index);
             } else {
                 match state.next_symbol().unwrap() {
                     Symbol::Rule(name) => {
-                        let rule = rules
-                            .iter()
-                            .find(|rule| rule.name == name)
-                            .unwrap();
+                        let rule = grammar.rules.get(&name).unwrap();
                         predict(&mut columns[column_index], rule);
                     }
                     Symbol::Lexeme(kind) => {
-                        if column_index + 1 < columns.len() {
-                            scan(&mut columns[column_index + 1], &state, &kind);
+                        if column_index + 1 < columns.len()
+                            && kind == columns[column_index + 1].kind
+                        {
+                            scan(&mut columns, column_index, state_index);
                         }
                     }
                 }
@@ -126,8 +145,12 @@ pub fn parse(
     }
 
     for column in columns.iter_mut() {
-        column.states =
-            column.states.iter().filter(|c| c.completed()).cloned().collect();
+        column.states = column
+            .states
+            .iter()
+            .filter(|state| state.completed())
+            .cloned()
+            .collect();
     }
 
     // println!();
@@ -141,7 +164,7 @@ pub fn parse(
 
     for state in &columns.last().unwrap().states {
         if state.name == START_RULE_NAME && state.completed() {
-            return Ok(build_forest(rules, lexemes, &columns, state));
+            return Ok(build_forest(grammar, lexemes, &columns, state));
         }
     }
 
